#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2022 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""
socket连接的工具类

@module socket
@file socket.py
"""
import sys
import os
import socket
import asyncio
import datetime
from HiveNetCore.generic import CResult, NullObj
from HiveNetCore.utils.exception_tool import ExceptionTool
from HiveNetCore.utils.run_tool import AsyncTools
# 根据当前文件路径将包路径纳入, 在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(
    os.path.dirname(__file__), os.path.pardir, os.path.pardir)))


class SocketTool(object):
    """
    socket连接的工具类
    """

    @classmethod
    def connect(cls, connect_config={}) -> CResult:
        """
        连接Socket服务器端并返回连接对象

        @param {dict} connect_config - 连接参数
            ip {str} - 主机名或IP地址, 默认为'127.0.0.1'
            port {int} - 监听端口, 默认为8080
            recv_timeout {float} - 数据接收的超时时间, 单位为秒, 默认为10.0
            send_timeout {float} - 数据发送的超时时间, 单位为秒, 默认为10.0

        @returns {CResult} - 连接结果:
            result.code: '00000'-成功, 其他值为失败
            result.net_info: 连接后的网络信息对象
                net_info.csocket - socket对象
                net_info.laddr 本地地址,地址对象("IP地址",打开端口)
                net_info.raddr 远端地址,地址对象("IP地址",打开端口)
                net_info.send_timeout 发送超时时间, 单位为秒
                net_info.recv_timeout 收取超时时间, 单位为秒
        """
        # 子类必须定义该功能
        _result = CResult('00000')
        _result.net_info = None
        with ExceptionTool.ignored_cresult(
            _result, logger=None
        ):
            # 进行连接
            _tcp_cli_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  # 分配 TCP 客户端套接字
            _tcp_cli_sock.connect(
                (connect_config.get('ip', '127.09.0.1'), connect_config.get('port', 8080))
            )  # 主动连接
            _tcp_cli_sock.setblocking(False)   # 将socket设置为非阻塞. 在创建socket对象后就进行该操作.

            # 处理网络连接对象
            _result.net_info = NullObj()
            _result.net_info.csocket = _tcp_cli_sock
            _result.net_info.laddr = _tcp_cli_sock.getsockname()
            _result.net_info.raddr = _tcp_cli_sock.getpeername()
            _result.net_info.send_timeout = connect_config.get('send_timeout', 10.0)
            _result.net_info.recv_timeout = connect_config.get('recv_timeout', 10.0)

        return _result

    @classmethod
    def close(cls, net_info: NullObj):
        """
        关闭指定的网络连接

        @param {NullObj} net_info - 需要关闭的网络连接信息对象
            net_info.csocket - socket对象
            net_info.laddr 本地地址,地址对象("IP地址",打开端口)
            net_info.raddr 远端地址,地址对象("IP地址",打开端口)
            net_info.send_timeout 发送超时时间, 单位为秒
            net_info.recv_timeout 收取超时时间, 单位为秒

        @returns {CResult} - 关闭结果
            result.code: '00000'-成功, 其他值为失败
        """
        _result = CResult('00000')
        with ExceptionTool.ignored_cresult(
            _result, logger=None
        ):
            net_info.csocket.close()

        return _result

    @classmethod
    def recv_data(cls, net_info, recv_para={}):
        """
        从指定的网络连接中读取数据

        @param {NullObj} net_info - 网络连接信息对象
            net_info.csocket - socket对象
            net_info.laddr 本地地址,地址对象("IP地址",打开端口)
            net_info.raddr 远端地址,地址对象("IP地址",打开端口)
            net_info.send_timeout 发送超时时间, 单位为秒
            net_info.recv_timeout 收取超时时间, 单位为秒
        @param {dict} recv_para - 读取数据的参数, 包括:
            recv_len {int} - 要获取的数据长度, 必要参数
            overtime {float} - 获取超时时间, 单位为秒, 非必要参数

        @returns {CResult} - 数据获取结果:
            result.code: '00000'-成功, '20403'-获取数据超时, 其他为获取失败
            result.data: 获取到的数据对象(具体类型和定义, 由实现类自定义)
            result.recv_time : datetime 实际开始接受数据时间
            result.overtime : float 超时时间(秒), 当返回结果为超时, 可获取超时时间信息
        """
        if type(recv_para) != dict:
            recv_para = {}

        _result = CResult('00000')
        _result.data = b''
        _result.recv_time = datetime.datetime.now()
        _overtime = recv_para.get('overtime', None)
        if _overtime is None:
            if hasattr(net_info, 'recv_timeout'):
                _overtime = net_info.recv_timeout
            else:
                _overtime = 10.0
        _result.overtime = _overtime

        with ExceptionTool.ignored_cresult(
            _result
        ):
            _rest_bytes = recv_para['recv_len']
            while _rest_bytes > 0:
                # 检查是否超时
                if (datetime.datetime.now() - _result.recv_time).total_seconds() > _overtime:
                    # 已超时
                    _result.change_code(code='20403')
                    break

                _buffer = b''
                with ExceptionTool.ignored(expect=(BlockingIOError)):
                    # 获取数据
                    _buffer = net_info.csocket.recv(_rest_bytes)
                    if _buffer is not None and len(_buffer) > 0:
                        _result.data = _result.data + _buffer
                        _rest_bytes = _rest_bytes - len(_buffer)
                    else:
                        # 休眠一下
                        AsyncTools.sync_run_coroutine(
                            asyncio.sleep(0.001)
                        )

        return _result

    @classmethod
    def send_data(cls, net_info, data: bytes, send_para={}):
        """
        向指定的网络连接发送数据

        @param {NullObj} net_info - 网络连接信息对象
            net_info.csocket - socket对象
            net_info.laddr 本地地址,地址对象("IP地址",打开端口)
            net_info.raddr 远端地址,地址对象("IP地址",打开端口)
            net_info.send_timeout 发送超时时间, 单位为秒
            net_info.recv_timeout 收取超时时间, 单位为秒
        @param {bytes} data - 要写入的数据对象
        @param {dict} send_para - 写入数据的参数:
            overtime {float} - 发送超时时间, 单位为秒, 非必须参数
        @returns {CResult} - 发送结果:
            result.code: '00000'-成功, '20404'-写入数据超时, 其他为写入失败
            result.send_time : datetime 实际发送完成时间
            result.overtime : float 超时时间(秒), 当返回结果为超时, 可获取超时时间信息
        """
        if type(send_para) != dict:
            send_para = {}

        _result = CResult('00000')
        _result.send_time = None
        _overtime = send_para.get('overtime', None)
        if _overtime is None:
            if hasattr(net_info, 'send_timeout'):
                _overtime = net_info.send_timeout
            else:
                _overtime = 10.0

        _result.overtime = _overtime

        _begin_time = datetime.datetime.now()
        with ExceptionTool.ignored_cresult(
            _result
        ):
            _rest_bytes = len(data)
            _total_bytes = _rest_bytes
            while _rest_bytes > 0:
                # 检查是否超时
                if (datetime.datetime.now() - _begin_time).total_seconds() > _overtime:
                    # 已超时
                    _result.change_code(code='20404')
                    break
                with ExceptionTool.ignored(expect=(BlockingIOError)):
                    # 发送数据
                    _len = net_info.csocket.send(data[_total_bytes - _rest_bytes:])
                    if _len > 0:
                        _rest_bytes = _rest_bytes - _len

            _result.send_time = datetime.datetime.now()
        return _result



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
连接池处理框架
@module connection_pool
@file connection_pool.py
"""

import os
import sys
import threading
import time
import asyncio
from logging import Logger
import traceback
from typing import Any
from HiveNetCore.utils.run_tool import AsyncTools
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))


__MOUDLE__ = 'connection_pool'  # 模块名
__DESCRIPT__ = u'连接池处理框架'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2022.04.22'  # 发布日期


class TooManyConnections(Exception):
    """
    连接数已太多抛出的异常
    """
    pass


class AIOConnectionPool(object):
    """
    支持异步模式的连接池处理框架
    抽象连接池的公共方法形成框架, 并提供基本的处理功能, 简化连接池编程的难度
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, creator: Any, pool_connection_class: Any, args: list = [], kwargs: dict = {}, connect_method_name: str = 'connect',
            max_size: int = 100, min_size: int = 0, connect_on_init: bool = False,
            blocking: bool = True, blocking_interval: float = 0.1, get_timeout: float = 10, free_idle_time: float = 30,
            ping_on_get: bool = False, ping_on_back: bool = False, ping_on_idle: bool = True,
            ping_interval: float = 20, ping_args: list = [], ping_kwargs: dict = {},
            daemon_interval: float = 0.1, pool_extend_paras: dict = {}, logger: Logger = None):
        """
        初始化连接池

        @param {Any} creator - 连接创建模块或对象, 结合connect_method_name、args、kwargs创建连接
            如果connect_method_name不为None: conn = creator.connect_method_name(*args, **kwargs)
            如果connect_method_name为None: conn = creator(*args, **kwargs)
        @param {Any} pool_connection_class - 连接池连接对象实现类(继承PoolConnectionFW的类对象)
        @param {list} args=[] - 进行连接创建的固定位置参数
        @param {dict} kwargs={} - 进行连接创建的kv参数
        @param {str} connect_method_name='connect' - 连接创建模块要执行的连接方法, 传None代表直接使用creator创建连接
        @param {int} max_size=100 - 连接池的最大连接数
        @param {int} min_size=0 - 连接池中最少保持的连接数(空闲也不删除)
        @param {bool} connect_on_init=False - 是否在初始化时创建一个连接
        @param {bool} blocking=True - 当获取不到连接时是否阻塞等待, 如果为False则代表直接抛出异常
        @param {float} blocking_interval=0.1 - 获取连接阻塞时的循环间隔时长, 单位为秒
        @param {float} get_timeout=10 - 等待连接获取的超时时间, 单位为秒, 0或None代表永不超时
        @param {float} free_idle_time=30 - 释放空闲连接的时间, 单位为秒, 0或None代表永不释放
        @param {bool} ping_on_get=False - 是否在外部获取连接时先检查连接是否有效(注: 在ping_interval时间内不会检查)
        @param {bool} ping_on_back=False - 是否在外部发挥连接时检查连接是否有效(注: 在ping_interval时间内不会检查)
        @param {bool} ping_on_idle=True - 是否对空闲连接进行是否有效的检查
        @param {float} ping_interval=20 - 空闲连接有效检查的间隔时长, 单位为秒
        @param {list} ping_args=[] - 检查连接有效性的函数固定位置参数
        @param {dict} ping_kwargs={} - 检查连接有效性的函数kv参数
        @param {float} daemon_interval=1 - 守护程序的循环间隔时长, 单位为秒
        @param {dict} pool_extend_paras={} - 连接池的扩展参数, 可传递到连接对象使用的个性参数
        @param {Logger} logger=None - 日志对象
        """
        # 进行参数处理
        self._creator = creator  # 该变量直接就是连接方法
        self._pool_connection_class = pool_connection_class
        self._args = args
        self._kwargs = kwargs
        self._connect_method_name = connect_method_name
        if self._connect_method_name is not None:
            self._creator = getattr(creator, self._connect_method_name)
        self._min_size = max(1, min_size)
        self._max_size = max(1, min_size, max_size)
        self._blocking = blocking
        self._blocking_interval = blocking_interval
        self._get_timeout = 0 if get_timeout is None else max(0, get_timeout)
        self._free_idle_time = 0 if free_idle_time is None else max(0, free_idle_time)
        self._ping_on_get = ping_on_get
        self._ping_on_back = ping_on_back
        self._ping_on_idle = ping_on_idle
        self._ping_interval = 0 if ping_interval is None else max(0, ping_interval)
        self._ping_args = ping_args
        self._ping_kwargs = ping_kwargs
        self._daemon_interval = daemon_interval
        self._pool_extend_paras = pool_extend_paras
        self._logger = logger

        # 内部的控制变量
        self._is_closed = False  # 指示连接池被关闭的标识
        self._lock = threading.RLock()  # 控制连接对象获取的多线程锁
        self._size = 0  # 当前线程池的总线程数
        self._conn_cached = []  # 空闲连接缓存数组

        # 线程池的守护线程, 处理线程检查、释放等处理
        if self._free_idle_time > 0 or self._ping_on_idle:
            self._daemon_thread = threading.Thread(
                target=self.__start_daemon_thread_fun,
                args=(1,),
                name='DaemonThread-ConnectionPool'
            )
            self._daemon_thread.setDaemon(True)
            self._daemon_running = True
            self._daemon_thread.start()

        # 初始化连接
        if connect_on_init:
            self._lock.acquire()
            try:
                _conn = AsyncTools.sync_run_coroutine(self._create_connection())
                self._conn_cached.append(_conn)
                self._size += 1
            finally:
                self._lock.release()

    #############################
    # 属性
    #############################
    @property
    def current_size(self):
        """
        获取连接池当前大小

        @property {int} - 连接池当前大小
        """
        return self._size

    #############################
    # 公共函数
    #############################
    async def close(self):
        """
        关闭连接池
        """
        self._is_closed = True
        # 释放所有连接
        while self._size > 0:
            self._lock.acquire()
            try:
                _conn = self._conn_cached.pop()
                self._size -= 1
                # 关闭连接
                try:
                    await _conn._final_close()
                except:
                    pass
            except IndexError:
                # 没有空闲的连接, 等待下一次获取
                await asyncio.sleep(0.1)
            finally:
                self._lock.release()

    async def connection(self):
        """
        获取一个有效连接
        """
        _start_time = time.time()
        while True:
            # 尝试获取连接
            self._lock.acquire()
            try:
                try:
                    _conn = self._conn_cached.pop()
                except IndexError:
                    # 没有空闲的连接, 尝试创建新连接然后重新获取
                    _conn = None
                else:
                    # 获取到连接, 进行检查
                    if self._ping_on_get and self._ping_interval > 0 and (time.time() - _conn.last_ping) >= self._ping_interval:
                        if not await _conn.ping(*self._ping_args, **self._ping_kwargs):
                            # 连接已失效, 直接丢弃连接
                            _conn = None
                            self._size -= 1

                if _conn is not None:
                    # 返回连接
                    return _conn

                # 判断是否允许创建新连接
                if self._size >= self._max_size:
                    if self._blocking:
                        if self._get_timeout == 0 or (time.time() - _start_time) < self._get_timeout:
                            # 等待连接释放
                            await asyncio.sleep(self._blocking_interval)
                            continue
                        else:
                            raise TooManyConnections('Too many connetions')
                    else:
                        # 不阻塞, 直接抛出异常
                        raise TooManyConnections('Too many connetions')
                else:
                    # 创建一个新连接, 并直接返回
                    _conn = await self._create_connection()
                    self._size += 1
                    return _conn
            finally:
                self._lock.release()

    #############################
    # 内部函数
    #############################
    async def _create_connection(self) -> Any:
        """
        创建一个新连接

        @returns {Any} - 返回创建的PoolConnectionFW实现类对象
        """
        return self._pool_connection_class(
            self, self._creator, self._args, self._kwargs
        )

    async def back_to_pool(self, conn: Any):
        """
        将完成使用的连接归还到连接池

        @param {Any} conn - PoolConnectionFW实现对象
        """
        self._lock.acquire()
        try:
            # 是否检查连接有效性
            if self._ping_on_back and self._ping_interval > 0 and (time.time() - conn.last_ping) >= self._ping_interval:
                if not await conn.ping(*self._ping_args, **self._ping_kwargs):
                    # 连接已无效
                    self._size -= 1
                    return

            # 重新放回连接池
            self._conn_cached.append(conn)
        finally:
            self._lock.release()

    #############################
    # 守护线程
    #############################
    def __start_daemon_thread_fun(self, tid):
        """
        守护线程, 负责检查连接有效性和释放连接

        @param {int} tid - 线程id
        """
        while True:
            if self._is_closed:
                # 出现关闭标记，退出守护
                break

            # 检查需要释放的空闲连接
            self._lock.acquire()
            try:
                _cache_size = len(self._conn_cached)
                _remove_count = self._min_size - _cache_size
                if self._free_idle_time > 0 and _remove_count > 0:
                    while _remove_count > 0:
                        if (time.time() - self._conn_cached[0].last_back) > self._free_idle_time:
                            # 达到空闲释放时间
                            _conn = self._conn_cached.pop(0)
                            self._size -= 1
                            _remove_count -= 1
                            # 关闭连接
                            try:
                                AsyncTools.sync_run_coroutine(_conn._final_close())
                            except:
                                # 记录日志
                                if self._logger is not None:
                                    self._logger.warning(
                                        'close connection error: %s' % traceback.format_exc()
                                    )
                            # 继续检查下一个
                            continue
                        else:
                            # 第一个达不到时间, 其他返回的时间肯定更短, 无需再判断
                            break
            except:
                pass
            finally:
                self._lock.release()

            # 检查连接的有效性, 从后往前检查, 此外为了避免检查导致获取连接的阻塞, 采用逐个检查的方式
            if self._ping_on_idle:
                _index = len(self._conn_cached)
                while _index > 0:
                    self._lock.acquire()
                    try:
                        if len(self._conn_cached) < _index:
                            # 当前长度小于要获取的序号, 有可能检查对象被取出了, 继续检查前一个
                            _index -= 1
                            continue

                        _conn = self._conn_cached[_index - 1]
                        if (time.time() - _conn.last_ping) >= self._ping_interval:
                            # 需要进行检查
                            if not AsyncTools.sync_run_coroutine(_conn.ping(*self._ping_args, **self._ping_kwargs)):
                                # 连接已失效, 直接从连接池取出
                                self._conn_cached.pop(_index - 1)
                                self._size -= 1
                    except:
                        # 记录日志
                        if self._logger is not None:
                            self._logger.warning(
                                'connection ping error: %s' % traceback.format_exc()
                            )
                    finally:
                        self._lock.release()

                    # 继续检查下一个
                    _index -= 1

            # 等待下一次处理
            AsyncTools.sync_run_coroutine(asyncio.sleep(self._daemon_interval))


class PoolConnectionFW(object):
    """
    连接池的通用连接对象框架(封装实际的连接对象)
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, pool: AIOConnectionPool, creator: Any, args: list = [], kwargs: dict = {},
            logger: Logger = None):
        """
        构造函数

        @param {AIOConnectionPool} pool - 创建该连接的连接池对象
        @param {Any} creator - 连接创建模块或对象, 结合connect_method_name、args、kwargs创建连接
            如果connect_method_name不为None: conn = creator.connect_method_name(*args, **kwargs)
            如果connect_method_name为None: conn = creator(*args, **kwargs)
        @param {list} args=[] - 进行连接创建的固定位置参数
        @param {dict} kwargs={} - 进行连接创建的kv参数
        @param {Logger} logger=None - 日志对象
        """
        self._pool = pool
        self._logger = logger
        # 创建连接
        self._conn = AsyncTools.sync_run_coroutine(
            creator(*args, **kwargs)
        )
        self.last_ping = time.time()  # 记录上次检查的时间
        self.last_back = time.time()  # 记录上次返回连接池的时间

    #############################
    # 通过重写__getattr__把真实连接对象的属性和函数绑定在当前类
    #############################
    def __getattr__(self, name):
        """
        获取函数的属性和函数

        @param {str} name - 属性名
        """
        return getattr(self._conn, name)

    #############################
    # 通用函数
    #############################
    async def ping(self, *args, **kwargs) -> bool:
        """
        连接有效性的检查函数

        @returns {bool} - 返回检查结果
        """
        try:
            _ping_result = await AsyncTools.async_run_coroutine(self._real_ping(*args, **kwargs))
            if _ping_result:
                self.last_ping = time.time()

            return _ping_result
        except:
            # 出现异常视为失败
            if self._logger is not None:
                self._logger.warning(
                    'pool_connection ping error: %s' % traceback.format_exc()
                )
            return False

    async def close(self):
        """
        关闭连接
        """
        # 不是真实关闭, 而是返回连接池
        self._conn = await AsyncTools.async_run_coroutine(self._fade_close())
        await self._pool.back_to_pool(self)

    #############################
    # 内部函数
    #############################
    async def _final_close(self):
        """
        真正进行连接的关闭
        """
        await AsyncTools.async_run_coroutine(
            self._real_close()
        )

    #############################
    # 需要继承类实现的函数
    #############################
    async def _real_ping(self, *args, **kwargs) -> bool:
        """
        实现类的真实检查连接对象是否有效的的函数

        @returns {bool} - 返回检查结果
        """
        raise NotImplementedError()

    async def _fade_close(self) -> Any:
        """
        实现类提供的虚假关闭函数
        注1: 不关闭连接, 只是清空上一个连接使用的上下文信息(例如数据库连接进行commit或rollback处理)
        注2: 如果必须关闭真实连接, 则可以关闭后创建一个新连接返回

        @returns {Any} - 返回原连接或新创建的连接
        """
        raise NotImplementedError()

    async def _real_close(self):
        """
        实现类提供的真实关闭函数
        """
        raise NotImplementedError()


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名: %s  -  %s\n'
           '作者: %s\n'
           '发布日期: %s\n'
           '版本: %s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2022 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""
服务鉴权模块

@module auth
@file auth.py
"""
import sys
import os
import copy
import re
import json
import datetime
from typing import Any
from functools import wraps
from HiveNetCore.utils.run_tool import AsyncTools
from HiveNetCore.utils.string_tool import StringTool
# 根据当前文件路径将包路径纳入, 在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))
from HiveNetWebUtils.utils.cryptography import HCrypto


class AuthBaseFw(object):
    """
    服务鉴权基础框架
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, **kwargs):
        """
        模块初始化函数
        (需由实现类继承实现)
        """
        pass

    #############################
    # 通用鉴权的修饰符处理函数
    #############################

    def auth_required(self, f=None):
        """
        当前模块的鉴权修饰符函数

        @param {function} f=None - 所执行的函数
        """
        def auth_required_internal(f):
            @wraps(f)
            def decorated(*args, **kwargs):
                # 执行实际处理函数函数
                return AsyncTools.sync_run_coroutine(self.auth_required_call(f, *args, **kwargs))
            return decorated

        if f:
            return auth_required_internal(f)
        return auth_required_internal

    async def auth_required_call(self, f, *args, **kwargs):
        """
        直接执行的鉴权修饰符函数

        @param {function} f - 要执行的函数
        @param {args} - 执行函数的固定入参
        @param {kwargs} - 执行函数的kv入参

        @returns {Any} - 返回响应信息(如果执行成功返回函数返回信息, 如果执行失败返回验证失败信息)
        """
        # 执行校验操作
        _result = await AsyncTools.async_run_coroutine(self._auth_call(*args, **kwargs))
        _is_auth_result = False
        if not _result[0]:
            # 校验失败, 获取校验失败的返回值
            _ret = await AsyncTools.async_run_coroutine(
                self._format_auth_resp(_result[1], _result[2])
            )
            _is_auth_result = True
        else:
            # 校验通过, 执行函数
            _ret = await AsyncTools.async_run_coroutine(f(*args, **kwargs))

        # 格式化响应对象并返回处理结果
        _last_ret = await AsyncTools.async_run_coroutine(
            self._format_last_resp(_ret, _is_auth_result)
        )
        return _last_ret

    #############################
    # 需实现类重载的函数
    #############################

    async def _auth_call(self, *args, **kwargs) -> tuple:
        """
        真正的校验处理函数

        @param {args} - 执行函数的固定入参
        @param {kwargs} - 执行函数的kv入参

        @returns {tuple} - 返回校验结果数组: (校验是否通过true/false, 错误码, 失败描述)
            注: 错误吗由实现类自行定义
        """
        raise NotImplementedError()

    def _format_auth_resp(self, code: Any, err_msg: str) -> Any:
        """
        格式化校验结果返回值

        @param {Any} code - 错误码
        @param {str} err_msg - 失败描述

        @returns {Any} - 格式化后的返回值
        """
        raise NotImplementedError()

    def _format_last_resp(self, resp: Any, is_auth_result: bool) -> Any:
        """
        格式化最后的响应对象

        @param {Any} resp - 最后的响应对象
        @param {bool} is_auth_result - 是否服务鉴权所返回的结果

        @returns {Any} - 转换以后的响应对象
        """
        return resp


class IPAuth(AuthBaseFw):
    """
    IP黑白名单模式验证模块
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, **kwargs):
        """
        IP黑白名单模式验证模块

        @param {list} init_blacklist=None - 初始化的黑名单
            名单可以使用通配符禁止某个网段, 例如 ['127.0.*.*', '138.*.*.*']
        @param {list} init_whitelist=None - 初始化的白名单
        @param {int} error_resp_status=403 - 验证失败返回状态码
        @param {str|dict} error_resp_msg={'status': '10409', 'msg':'IP地址验证失败'} - 验证失败返回的信息
        """
        self.para = kwargs
        self.error_resp_status = self.para.get('error_resp_status', 403)
        self.error_resp_msg = self.para.get(
            'error_resp_msg', {'status': '10409', 'msg': 'IP地址验证失败'}
        )

        # 黑白名单管理
        self.ip_dict = {
            'blacklist': {
                'show': list(),  # 显示配置
                'reg': dict()  # 正则表的配置, key为show中的显示名, value为正则表达式规则
            },
            'whitelist': {
                'show': list(),  # 显示配置
                'reg': dict()  # 正则表的配置, key为show中的显示名, value为正则表达式规则
            }
        }

        # 初始化黑白名单
        if kwargs.get('init_blacklist', None) is not None:
            self.add_blacklist(kwargs['init_blacklist'])

        if kwargs.get('init_whitelist', None) is not None:
            self.add_whitelist(kwargs['init_whitelist'])

    #############################
    # 黑白名单检查工具
    #############################

    def verify_blacklist(self, ip: str) -> bool:
        """
        验证是否匹配黑名单

        @param {str} ip - 要验证的ip

        @returns {bool} - 检查结果, 匹配到返回True
        """
        return self._verify_ip('blacklist', ip)

    def verify_whitelist(self, ip: str) -> bool:
        """
        验证是否匹配白名单

        @param {str} ip - 要验证的ip

        @returns {bool} - 检查结果, 匹配到返回True
        """
        return self._verify_ip('whitelist', ip)

    #############################
    # 黑白名单维护工具
    #############################

    def add_blacklist(self, ips):
        """
        添加黑名单

        @param {str|list} ips - 要添加的ip或ip列表
        """
        if type(ips) == str:
            # 单个黑名单
            self._add_list('blacklist', ips)
        else:
            # 多个黑名单
            for _ip in ips:
                self._add_list('blacklist', _ip)

    def remove_blacklist(self, ips):
        """
        删除黑名单

        @param {str|list} ips - 要删除的ip或ip列表
        """
        if type(ips) == str:
            # 单个黑名单
            self._remove_list('blacklist', ips)
        else:
            # 多个黑名单
            for _ip in ips:
                self._remove_list('blacklist', _ip)

    def clear_blacklist(self):
        """
        清除黑名单
        """
        self.ip_dict['blacklist'].clear()
        self.ip_dict['blacklist']['show'] = list()
        self.ip_dict['blacklist']['reg'] = dict()

    def add_whitelist(self, ips):
        """
        添加白名单

        @param {str|list} ips - 要添加的ip或ip列表
        """
        if type(ips) == str:
            # 单个黑名单
            self._add_list('whitelist', ips)
        else:
            # 多个黑名单
            for _ip in ips:
                self._add_list('whitelist', _ip)

    def remove_whitelist(self, ips):
        """
        删除白名单

        @param {str|list} ips - 要删除的ip或ip列表
        """
        if type(ips) == str:
            # 单个黑名单
            self._remove_list('whitelist', ips)
        else:
            # 多个黑名单
            for _ip in ips:
                self._remove_list('whitelist', _ip)

    def clear_whitelist(self):
        """
        清除白名单
        """
        self.ip_dict['whitelist'].clear()
        self.ip_dict['whitelist']['show'] = list()
        self.ip_dict['whitelist']['reg'] = dict()

    #############################
    # 内部函数
    #############################

    def _add_list(self, ip_type: str, ip: str):
        """
        插入名单数据

        @param {str} ip_type - 'blacklist' 或 'whitelist'
        @param {str} ip - 要插入的ip地址
        """
        _dict = self.ip_dict[ip_type]
        if ip in _dict['show']:
            # 名单已存在
            return

        # 加入显示名单
        _dict['show'].append(ip)

        # 生成名单的匹配正则表达式
        if ip.find('*') >= 0:
            # 需要生成正则表达式
            _re = re.compile('^' + ip.replace('.', '\\.').replace('*', '.*') + '$')
            _dict['reg'][ip] = _re

    def _remove_list(self, ip_type: str, ip: str):
        """
        删除名单数据

        @param {str} ip_type - 'blacklist' 或 'whitelist'
        @param {str} ip - 要删除的ip地址
        """
        _dict = self.ip_dict[ip_type]
        if ip in _dict['show']:
            # 删除正则表达式
            _dict['reg'].pop(ip, None)

            # 删除显示ip
            _dict['show'].remove(ip)

    def _verify_ip(self, ip_type: str, ip: str) -> bool:
        """
        检查ip是否匹配名单

        @param {str} ip_type - 'blacklist' 或 'whitelist'
        @param {str} ip - 要检查的ip地址

        @returns {bool} - 检查结果, 匹配到返回True
        """
        _dict = self.ip_dict[ip_type]
        if ip in _dict['show']:
            return True

        # 遍历正则规则
        for _re in _dict['reg'].values():
            if _re.search(ip) is not None:
                return True

        # 没有匹配上
        return False

    #############################
    # 重载基础框架的函数
    #############################

    async def _auth_call(self, *args, **kwargs) -> tuple:
        """
        真正的校验处理函数

        @param {args} - 执行函数的固定入参
        @param {kwargs} - 执行函数的kv入参

        @returns {tuple} - 返回校验结果数组: (校验是否通过true/false, 错误码, 失败描述)
            错误码定义如下: 200-成功, 其他-失败
        """
        _status = 200
        _ip = await AsyncTools.async_run_coroutine(self._get_ip_from_request(*args, **kwargs))

        # 先检查白名单
        if len(self.ip_dict['whitelist']['show']) > 0:
            if not self.verify_whitelist(_ip):
                # 不在白名单内
                _status = self.error_resp_status
                _resp_msg = copy.deepcopy(self.error_resp_msg)

        # 再检查黑名单
        if _status == 200 and self.verify_blacklist(_ip):
            # 在黑名单内
            _status = self.error_resp_status
            _resp_msg = copy.deepcopy(self.error_resp_msg)

        if _status == 200:
            # 返回校验成功
            return (True, _status, 'success')
        else:
            # 校验失败
            return (False, _status, _resp_msg)

    #############################
    # 需实现类重载的函数
    #############################

    def _format_auth_resp(self, code: Any, err_msg: str) -> Any:
        """
        格式化校验结果返回值

        @param {Any} code - 错误码
        @param {str} err_msg - 失败描述

        @returns {Any} - 格式化后的返回值
        """
        raise NotImplementedError()

    def _get_ip_from_request(self, *args, **kwargs) -> str:
        """
        从请求信息中获取ip地址

        @returns {str} - 返回ip地址
        """
        raise NotImplementedError()


class AppKeyAuth(AuthBaseFw):
    """
    AppKey模式验证模块
    整体流程: 调用方对请求报文签名 -> 服务方验证请求签名 -> 服务方处理并对返回报文签名 -> 调用方验证返回报文签名
    详细说明如下:
        1. 服务端生成APP信息, 线下提供给商户
            AppId: 商户id
            AppKey: 公匙(相当于账号)
            AppSecret: 私匙(相当于密码)
        2. 客户端对要发送的数据进行签名, 算法如下:
        (1) 客户端生成 nonce_str 随机字符串, 例如: 'ibuaiVcKdpRxkhJA'
        (2) 设要发送的数据为集合M, 将所有非空参数值的参数按照参数名ASCII码从小到大排序(字典序),
        使用URL键值对的格式(即key1=value1&key2=value2…)拼接成字符串stringA, 例如:
            stringA="body=test&device_info=1000&mch_id=10000100"
        (3) 拼接API密钥
            # 拼接app_id、app_key、app_secret、nonce_str、timestamp进入签名字符串
            stringSignTemp=stringA+"&app_id=1333&app_key=123456&app_secret=192006250b4c09247ec02edce69f6a2d&nonce_str=xx&timestamp=xx"
            # 如果选择MD5签名方式, 处理及得到结果如下
            sign=MD5(stringSignTemp).toUpperCase()="9A0A8659F005D6984697E2CA0A9CF3B7"
            # 如果选择HMAC-SHA256算法签名方式, 处理及得到结果如下
            # 注意: 部分语言的hmac方法生成结果二进制结果, 需要调对应函数转化为十六进制字符串。
            sign=hash_hmac("sha256",stringSignTemp,AppSecret).toUpperCase()="6A9AE1657590FD6257D693A078E1C3E4BB6BA4DC30B23E0EE2496E54170DACD6"
        (4) 将sign放入要发送的数据集合中, 客户端调用api接口
        (5) 服务器端同样做相应的认证检查
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, **kwargs):
        """
        AppKey模式验证模块

        @param {fuction} get_secret_fun - 取 (app_key, app_secret) 密钥对的函数, 默认使用当前类的自有AppKey管理工具函数
            fun(app_id:str) -> tuple
        @param {bool} sign_resp=False - 是否对返回的报文进行签名
        @param {int} sign_error_resp_status=403 - 签名验证失败返回状态码
        @param {str|dict} sign_error_resp_msg={'status': '13007', 'msg':'签名检查失败'} - 签名验证失败返回的信息
        @param {float} timestamp_expired_time=300.0 - 允许服务器时间差异时长, 单位为秒, 默认5分钟
        @param {int} timestamp_error_resp_status=403 - timestamp已过期时返回状态码
        @param {str|dict} timestamp_error_resp_msg={'status': '13008', 'msg':'时间戳已过期'} - timestamp已过期返回的信息
        @param {int} nonce_len=8 - nonce字符串的长度
        @param {str} timestamp_fmt='%Y%m%d%H%M%S' - timestamp的格式
        @param {str} encoding='utf-8' - 对中文内容的转换编码
        @param {str} algorithm='MD5' - 使用的签名算法名, 支持算法如下
            MD5
            HMAC-SHA256
        @param {dict} algorithm_extend=None - 扩展算法支持, key为algorithm名, value为扩展的算法函数
            扩展函数定义如下: fun(value:str, key:str) -> str
        """
        self.para = kwargs
        self.get_secret_fun = self.para.get('get_secret_fun', self.apk_get_secret_fun)
        self.sign_resp = self.para.get('sign_resp', False)
        self.interface_id_name = self.para.get('interface_id_name', '')
        self.sign_error_resp_status = self.para.get('sign_error_resp_status', 403)
        self.sign_error_resp_msg = self.para.get(
            'sign_error_resp_msg', {'status': '13007', 'msg': '签名检查失败'}
        )
        self.timestamp_expired_time = self.para.get('timestamp_expired_time', 300.0)
        self.timestamp_error_resp_status = self.para.get('timestamp_error_resp_status', 403)
        self.timestamp_error_resp_msg = self.para.get(
            'timestamp_error_resp_msg', {'status': '13008', 'msg': '时间戳已过期'}
        )
        self.timestamp_fmt = self.para.get('timestamp_fmt', '%Y%m%d%H%M%S')
        self.encoding = self.para.get('encoding', 'utf-8')

        # 算法扩展支持
        self.algorithm_mapping = {
            'MD5': HCrypto.md5,
            'HMAC-SHA256': HCrypto.hmac_sha256
        }
        self.algorithm_mapping.update(self.para.get('algorithm_extend', {}))
        self.algorithm = self.para.get('algorithm', 'MD5')

        # 简易的AppKey管理台, 内存字典管理, key为app_id, value为(app_key, app_secret) 键值对
        self._app_key_manager = dict()

    #############################
    # 签名工具
    #############################
    def get_signature(self, msg: dict, app_key: str, app_secret: str, algorithm: str = None) -> str:
        """
        对消息字典进行签名

        @param {dict} msg - 要签名的字典
        @param {str} app_key - 商户持有的app_key(相当于公钥)
        @param {str} app_secret - 商户持有的AppSecret私匙(相当于密码)
        @param {str} algorithm=None - 使用的签名算法名, 如果不传使用初始化类的指定算法, 支持算法如下
            MD5
            HMAC-SHA256
        @returns {str} - 返回签名验证字符串
        """
        # 必须要有的参数
        _app_id = msg['app_id']
        _nonce_str = msg['nonce_str']
        _timestamp = msg['timestamp']

        # 参数清单组合, 按参数名排序, 去掉非空值, URL键值对方式组合
        _para_list = list(msg.keys())
        _para_list.sort()
        _str_sign_temp = ''
        for _para in _para_list:
            if _para not in ('app_id', 'nonce_str', 'timestamp', 'sign') and msg[_para] not in (None, ''):
                _value = msg[_para]
                if type(_value) != str:
                    _value = json.dumps(_value, ensure_ascii=False, sort_keys=True)

                _str_sign_temp = '%s%s=%s&' % (
                    _str_sign_temp, _para, _value
                )

        # 增加app_id、app_key、app_secret, nonce_str、timestamp到键值对中
        _str_sign_temp = '%sapp_id=%s&app_key=%s&app_secret=%s&nonce_str=%s&timestamp=%s' % (
            _str_sign_temp, _app_id, app_key, app_secret, _nonce_str, _timestamp
        )

        # 进行加密处理并返回签名串
        _algorithm = self.algorithm if algorithm is None else algorithm
        return self.algorithm_mapping[_algorithm](
            _str_sign_temp, key=app_secret, encoding=self.encoding
        )

    def sign(self, msg: dict) -> dict:
        """
        对报文消息字典进行签名

        @param {dict} msg - 要签名的报文字典

        @returns {dict} - 签名后的报文字典
        """
        _app_id = msg['app_id']
        msg['nonce_str'] = HCrypto.generate_nonce(self.para.get('nonce_len', 8))  # 随机字符串
        msg['timestamp'] = datetime.datetime.now().strftime(self.timestamp_fmt)  # 时间戳
        _sign_type = msg.get('sign_type', self.algorithm)  # 签名类型, 如果有送值代表指定算法
        _app_key, _app_secret = self.get_secret_fun(_app_id)  # 通过指定的算法获取
        msg['sign'] = self.get_signature(msg, _app_key, _app_secret, algorithm=_sign_type)

        return msg

    def verify_sign(self, msg: dict) -> bool:
        """
        验证报文签名是否准确

        @param {dict} msg - 要验证的报文字典

        @returns {bool} - 报文验证结果
        """
        try:
            _app_id = msg['app_id']
            _sign_type = msg.get('sign_type', self.algorithm)  # 签名类型, 如果有送值代表指定算法
            _app_key, _app_secret = self.get_secret_fun(_app_id)  # 通过指定的算法获取
            _sign = self.get_signature(msg, _app_key, _app_secret, algorithm=_sign_type)
            return _sign == msg['sign']
        except:
            return False

    def verify_timestamp(self, msg: dict) -> bool:
        """
        验证时间戳是否已过期

        @param {dict} msg - 要验证的报文字典

        @returns {bool} - 验证结果
        """
        try:
            _timestamp = datetime.datetime.strptime(msg['timestamp'], self.timestamp_fmt)
            if abs((datetime.datetime.now() - _timestamp).total_seconds()) > self.timestamp_expired_time:
                return False
            return True
        except:
            return False

    #############################
    # 简易AppKey管理台工具
    #############################
    def apk_get_secret_fun(self, app_id: str) -> tuple:
        """
        自有AppKey管理工具(无安全控制)的取密钥对函数

        @param {str} app_id - 要获取的app_id

        @returns {tuple} - (app_key, app_secret) 密钥对
        """
        return self._app_key_manager[app_id]

    def apk_update_secret(self, app_id: str, key_pair: tuple):
        """
        自有AppKey管理工具的密钥对更新

        @param {str} app_id - 要更新的app_id
        @param {tuple} key_pair - (app_key, app_secret) 密钥对
        """
        self._app_key_manager[app_id] = key_pair

    def apk_generate_key_pair(self, app_id: str) -> tuple:
        """
        自有AppKey管理工具的生成新密钥对函数(同时可以加入管理工具)

        @param {str} app_id - 要获取的app_id

        @returns {tuple} - (app_key, app_secret) 密钥对
        """
        # 随机生成字符串, app_key 8位, app_secret 32位
        _app_key = StringTool.get_random_str(random_length=8)
        _app_secret = StringTool.get_random_str(random_length=32)
        self._app_key_manager[app_id] = (_app_key, _app_secret)

        return (_app_key, _app_secret)

    #############################
    # 重载基础框架的函数
    #############################
    async def auth_required_call(self, f, *args, **kwargs):
        """
        直接执行的鉴权修饰符函数
        注: 进行了重载, 增加app_id的获取并送入_format_last_resp函数

        @param {function} f - 要执行的函数
        @param {args} - 执行函数的固定入参
        @param {kwargs} - 执行函数的kv入参

        @returns {Any} - 返回响应信息(如果执行成功返回函数返回信息, 如果执行失败返回验证失败信息)
        """
        # 执行校验操作
        _result = await AsyncTools.async_run_coroutine(self._auth_call(*args, **kwargs))
        _is_auth_result = False
        if not _result[0]:
            # 校验失败, 获取校验失败的返回值
            _ret = await AsyncTools.async_run_coroutine(
                self._format_auth_resp(_result[1], _result[2])
            )
            _is_auth_result = True
        else:
            # 校验通过, 执行函数
            _ret = await AsyncTools.async_run_coroutine(f(*args, **kwargs))

        # 格式化响应对象并返回处理结果
        _json = self._get_json_from_request(*args, **kwargs)
        _last_ret = await AsyncTools.async_run_coroutine(
            self._format_last_resp(_ret, _is_auth_result, _json['app_id'])
        )
        return _last_ret

    async def _auth_call(self, *args, **kwargs) -> tuple:
        """
        真正的校验处理函数

        @param {args} - 执行函数的固定入参
        @param {kwargs} - 执行函数的kv入参

        @returns {tuple} - 返回校验结果数组: (校验是否通过true/false, 错误码, 失败描述)
            错误码定义如下: 200-成功, 其他-失败
        """
        _status = 200
        _json_dict = await AsyncTools.async_run_coroutine(self._get_json_from_request(*args, **kwargs))

        if not self.verify_timestamp(_json_dict):
            # 日期验证失败
            _status = self.timestamp_error_resp_status
            _resp_msg = copy.deepcopy(self.timestamp_error_resp_msg)
        elif not self.verify_sign(_json_dict):
            # 验证失败, 返回标准的错误信息
            _status = self.sign_error_resp_status
            _resp_msg = copy.deepcopy(self.sign_error_resp_msg)

        if _status == 200:
            # 返回校验成功
            return (True, _status, 'success')
        else:
            # 校验失败
            return (False, _status, _resp_msg)

    def _format_auth_resp(self, code: Any, err_msg: str) -> Any:
        """
        格式化校验结果返回值

        @param {Any} code - 错误码
        @param {str} err_msg - 失败描述

        @returns {tuple} - 返回校验结果数组: (错误码, 失败描述)
            错误码定义如下: 200-成功, 其他-失败
        """
        return (code, err_msg)

    #############################
    # 需实现类重载的函数
    #############################

    def _format_last_resp(self, resp: Any, is_auth_result: bool, app_id: str) -> Any:
        """
        格式化最后的响应对象
        注意: 实现类需实现以下的伪代码逻辑
        if self.sign_resp:
            resp_dict = get resp_dict from resp
            put app_id to resp_dict
            resp_dict = self.sign(resp_dict)
            put resp_dict back to resp

        @param {Any} resp - 最后的响应对象
        @param {bool} is_auth_result - 是否服务鉴权所返回的结果
        @param {str} app_id - 送入的app_id

        @returns {Any} - 转换以后的响应对象
        """
        raise NotImplementedError()

    def _get_json_from_request(self, *args, **kwargs) -> dict:
        """
        从请求信息中获取消息json

        @returns {dict} - 返回json字典
        """
        raise NotImplementedError()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
通用的加解密处理公共模块

@module cryptography
@file cryptography.py
"""

import os
import sys
import hashlib
import hmac
import base64
if sys.platform != 'win32':
    from Crypto import Random
    from Crypto.PublicKey import RSA
    from Crypto.Cipher import PKCS1_v1_5
else:
    from Cryptodome import Random
    from Cryptodome.PublicKey import RSA
    from Cryptodome.Cipher import PKCS1_v1_5  # 注: JS加密的内容 PKCS1_OAEP 没有办法解密
from HiveNetCore.utils.string_tool import StringTool
# 根据当前文件路径将包路径纳入, 在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(
    os.path.dirname(__file__), os.path.pardir, os.path.pardir)))


__MOUDLE__ = 'cryptography'  # 模块名
__DESCRIPT__ = u'通用的加解密处理公共模块'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2020.12.08'  # 发布日期


class HCrypto(object):
    """
    通用加解密公共模块
    """

    #############################
    # 随机数产生函数
    #############################
    @classmethod
    def generate_salt(cls, str_len=8,
                      chars="AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz"):
        """
        随机生成盐字符串

        @param {int} str_len=8 - 盐长度
        @param {string} chars="AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz" - 生成盐包含的字符集合

        @returns {str} - 生成的盐字符串
        """
        return StringTool.get_random_str(
            random_length=str_len,
            chars=chars
        )

    @classmethod
    def generate_nonce(cls, length: int = 8,
                       chars="AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789") -> str:
        """
        生成nonce随机字符串

        @param {int} length=8 - 要生成的字符串长度
        @param {string} chars="AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz0123456789" - 随机抽取的字符串内容

        @returns {str} - 返回随机字符串
        """
        return StringTool.get_random_str(random_length=length, chars=chars)

    #############################
    # Hash散列算法
    #############################
    @classmethod
    def md5(cls, value, encoding='utf-8', **kwargs) -> str:
        """
        Md5加密算法

        @param {str|bytes} value - 要加密的字符串或bytes数组
        @param {str} encoding='utf-8' - 对value字符串的转换编码

        @returns {str} - 返回加密后的字符串
        """
        _value = value
        if type(value) == str:
            _value = value.encode(encoding=encoding)
        return hashlib.md5(_value).hexdigest().upper()

    @classmethod
    def sha1(cls, value: str, encoding='utf-8', **kwargs) -> str:
        """
        SHA1加密算法

        @param {str|bytes} value - 要加密的字符串或bytes数组
        @param {str} encoding='utf-8' - 对value字符串的转换编码

        @returns {str} - 返回加密后的字符串
        """
        _value = value
        if type(value) == str:
            _value = value.encode(encoding=encoding)
        return hashlib.sha1(_value).hexdigest().upper()

    @classmethod
    def sha256(cls, value: str, encoding='utf-8', **kwargs) -> str:
        """
        SHA256加密算法

        @param {str|bytes} value - 要加密的字符串或bytes数组
        @param {str} encoding='utf-8' - 对value字符串的转换编码

        @returns {str} - 返回加密后的字符串
        """
        _value = value
        if type(value) == str:
            _value = value.encode(encoding=encoding)
        return hashlib.sha256(_value).hexdigest().upper()

    @classmethod
    def sha512(cls, value: str, encoding='utf-8', **kwargs) -> str:
        """
        SHA512加密算法

        @param {str|bytes} value - 要加密的字符串或bytes数组
        @param {str} encoding='utf-8' - 对value字符串的转换编码

        @returns {str} - 返回加密后的字符串
        """
        _value = value
        if type(value) == str:
            _value = value.encode(encoding=encoding)
        return hashlib.sha512(_value).hexdigest().upper()

    #############################
    # 不可逆加密算法
    #############################
    @classmethod
    def hmac_sha256(cls, value: str, key: str, encoding='utf-8') -> str:
        """
        HMAC-SHA256加密算法

        @param {str} value - 要加密的字符串内容
        @param {str} key - 加密密钥
        @param {str} encoding='utf-8' - 对value字符串的转换编码

        @returns {str} - 返回加密后的字符串
        """
        _hash_obj = hmac.new(
            key.encode(), value.encode(encoding=encoding), digestmod=hashlib.sha256
        )
        return _hash_obj.hexdigest().upper()

    #############################
    # RSA 加密
    #############################
    @classmethod
    def rsa_generate_key_pair(cls, bits: int = 1024, e: int = 65537, format: str = 'PEM',
                              pkcs: int = 1, passphrase: str = None) -> tuple:
        """
        生成RSA密钥对

        @param {int} bits=enum[*1024,2048,3072] - 密钥长度
        @param {int} e=65537 - 公钥指数, 最小为65537, 不建议修改
        @param {str} format=enum[*'PEM','DER','OpenSSH'] - 输出的密钥格式
            PEM - 文本编码格式, 满足标准 `RFC1421`/`RFC1423`
            DER - 二进制格式
            OpenSSH - 文本编码格式, 满足 OpenSSH 标准
        @param {int} pkcs=enum[*1,8] - 私钥的结构标准
            1 - 私钥编码为简单的 PKCS#1 结构(RSAPrivateKey)
            2 - 私钥编码为 PKCS#8 结构(PrivateKeyInfo)
        @param {str} passphrase=None - 证书开启密码, 用于保护证书的输出内容

        @returns {(bytes, bytes)} - 返回 (私钥字节数组, 公钥字节数组)
        """
        _random_generator = Random.new().read
        _rsa = RSA.generate(bits, _random_generator, e=e)

        # 生成私钥
        _private_key = _rsa.exportKey(format=format, pkcs=pkcs, passphrase=passphrase)

        # 生成公钥
        _public_key = _rsa.publickey().exportKey(format=format, pkcs=pkcs)

        return (_private_key, _public_key)

    @classmethod
    def rsa_get_key(cls, extern_key, passphrase=None):
        """
        RSA获取密钥对象(公钥或私钥)

        @param {str|bytes} extern_key - 输入的密钥内容, 可以为字符串或bytes对象
        @param {str} passphrase=None - 密钥的开启密码

        @returns {Cryptodome.PublicKey.RSA.RsaKey} - 返回可以用于加解密的密钥对象
        """
        return RSA.import_key(extern_key, passphrase=passphrase)

    @classmethod
    def rsa_encrypt(cls, rsa_key, data, encoding: str = 'utf-8', output_str: bool = True):
        """
        RSA加密数据
        注意: RSA支持加密数据长度跟密钥长度有关,因此不建议使用RSA加密大数据

        @param {Cryptodome.PublicKey.RSA.RsaKey} rsa_key - 通过 rsa_get_key 获取到的 RsaKey 对象
        @param {str|bytes} data - 要加密的数据, 支持字符串或字节数据两种类型
        @param {str} encoding='utf-8' - 对字符串的转换编码
        @param {bool} output_str=True - 输出结果是否转换为字符串格式

        @returns {str|bytes} - 如果output_str为True, 则通过 base64.b64encode 转换为可打印字符输出, 否则返回二进制字节数组
        """
        _cipher_rsa = PKCS1_v1_5.new(rsa_key)

        # 进行数据转换
        _data = data
        if type(data) == str:
            _data = bytes(data, encoding=encoding)

        # 进行加密处理
        _encrypt_data = _cipher_rsa.encrypt(_data)

        # 将加密结果转换为 base64 编码字符串
        if output_str:
            return base64.b64encode(_encrypt_data).decode(encoding='ascii')
        else:
            return _encrypt_data

    @classmethod
    def ras_decrypt(cls, rsa_key, encrypt_data, output_str: bool = True, encoding: str = 'utf-8'):
        """
        解密数据

        @param {Cryptodome.PublicKey.RSA.RsaKey} rsa_key - 通过 rsa_get_key 获取到的 RsaKey 对象
        @param {str|bytes} encrypt_data - 要解密的数据, 支持 base64 字符串或字节数据两种类型
        @param {bool} output_str=True - 输出结果是否转换为真实字符串 (加密数据为字符串)
        @param {str} encoding='utf-8' - 数据结果转换为字符串的编码

        @returns {bytes|str} - 解密后的结果, 如果 output_str 为 True 则为字符串, 否则为二进制字节数组
        """
        _cipher_rsa = PKCS1_v1_5.new(rsa_key)

        # 进行数据处理
        _encrypt_data = encrypt_data
        if type(encrypt_data) == str:
            _encrypt_data = base64.b64decode(bytes(_encrypt_data, encoding='ascii'))

        # 解密
        _bytes_data = _cipher_rsa.decrypt(_encrypt_data, None)

        # 看是否转换为字符串
        if output_str:
            return _bytes_data.decode(encoding=encoding)
        else:
            return _bytes_data


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名: %s  -  %s\n'
           '作者: %s\n'
           '发布日期: %s\n'
           '版本: %s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2022 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
Web服务基础框架类

@module server
@file server.py
"""
import sys
import os
import logging
import copy
import threading
import datetime
import asyncio
import json
import socket
from functools import wraps
from collections import OrderedDict
from enum import Enum
from typing import Callable
from HiveNetCore.generic import CResult, NullObj
from HiveNetCore.i18n import _, get_global_i18n, init_global_i18n
from HiveNetCore.utils.exception_tool import ExceptionTool
from HiveNetCore.utils.net_tool import NetTool
from HiveNetCore.utils.run_tool import AsyncTools, RunTool
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))
from HiveNetWebUtils.utils.socket import SocketTool


__MOUDLE__ = 'server'  # 模块名
__DESCRIPT__ = u'Web服务基础框架类'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2022.05.04'  # 发布日期


class EnumServerRunStatus(Enum):
    """
    服务器运行状态

    @enum {string}
    """
    Stop = 'Stop'  # 停止
    Running = 'Running'  # 正在运行
    WaitStop = 'WaitStop'  # 等待停止
    WaitStart = 'WaitStart'  # 等待启动
    ForceStop = 'ForceStop'  # 强制停止


class ServerBaseFW(object):
    """
    服务基础类, 定义了服务的标准调用函数
    """
    #############################
    # 静态函数
    #############################
    @classmethod
    def get_init_server_dict(cls) -> OrderedDict:
        """
        获取已经初始化的服务访问字典

        @returns {OrderedDict} - 返回字典
        """
        INIT_SERVER = RunTool.get_global_var('INIT_HIVENET_WEB_UTILS_SERVER')
        if INIT_SERVER is None:
            INIT_SERVER = OrderedDict()
            RunTool.set_global_var('INIT_HIVENET_WEB_UTILS_SERVER', INIT_SERVER)

        return INIT_SERVER

    @classmethod
    def get_init_server(cls, app_name: str = None):
        """
        获取已经初始化的服务对象

        @param {str} app_name=None - 要获取的服务的app_name, 如果不传默认取第一个

        @returns {ServerBaseFW} - 获取到的对象
        """
        INIT_SERVER = cls.get_init_server_dict()
        if app_name is None:
            return list(INIT_SERVER.values())[0]
        else:
            return INIT_SERVER[app_name]

    @classmethod
    def get_auth_fun(cls, auth_name: str = '', app_name: str = None):
        """
        获取Server支持的Auth实例对象
        注: 该函数主要用于 auth_required_static 修饰符入参

        @param {str} auth_name='' - 验证对象类型名(比如类名, 具体取决于Server的初始化参数)
        @param {str} app_name=None - 要获取的Server的app_name, 如果不传默认取第一个
        """
        _server = cls.get_init_server(app_name=app_name)
        return _server._support_auths[auth_name]

    @classmethod
    def auth_required_static(cls, f=None, auth_name: str = '', app_name: str = None):
        """
        静态的服务鉴权修饰函数
        (用于支持鉴权对象实例化前对函数进行修饰处理)

        @param {function} f - 要调用的函数
        @param {str} auth_name='' - 验证对象类型名(比如类名, 具体取决于Server的初始化参数)
        @param {str} app_name=None - 要获取的Server的app_name, 如果不传默认取第一个

        @example  指定API需要IP黑白名单验证
            class RestfulApiClass(object):
                @classmethod
                @ServerBaseFW.auth_required_static(auth_name='IPAuth', app_name='demo_servr')
                def login(user_name, methods=['POST'], **kwargs):
                    ...
        """
        def auth_internal(f):
            @wraps(f)
            def decorated(*args, **kwargs):
                # 获取验证对象
                _auth = cls.get_auth_fun(auth_name=auth_name, app_name=app_name)

                # 执行验证修饰函数
                return _auth.auth_required_call(f, *args, **kwargs)
            return decorated

        if f:
            return auth_internal(f)
        return auth_internal

    #############################
    # 构造函数和析构函数
    #############################
    def __init__(self, app_name: str, server_config: dict = {}, support_auths: dict = {},
            before_server_start=None, after_server_start=None,
            before_server_stop=None, after_server_stop=None, logger=None, log_level: int = logging.INFO,
            load_i18n_para=None, **kwargs):
        """
        构造函数

        @param {str} app_name - 服务器名称
        @param {dict} server_config={} - 服务配置字典, 由实现类自定义传值, 例如绑定主机, 端口等
        @param {dict} support_auths={} - 服务器支持的验证对象字典, key为验证对象类型名(可以为类名), value为验证对象实例对象
            注意: 支持的auth对象必须有auth_required这个修饰符函数
        @param {function} before_server_start=None - 服务器启动前执行的函数对象, 传入服务自身(self)
        @param {function} after_server_start=None - 服务器启动后执行的函数对象, 传入服务自身(self)
        @param {function} before_server_stop=None - 服务器关闭前执行的函数对象, 传入服务自身(self)
        @param {function} after_server_stop=None - 服务器关闭后执行的函数对象, 传入服务自身(self)
        @param {Logger} logger=None - 自定义应用逻辑使用的日志对象
        @param {int} log_level=logging.INFO - 一般信息的记录使用的日志级别
        @param {dict} load_i18n_para=None - 要装载的i18n语言文件配置
            path {str} - 要加载的i18n字典文件路径, 如果填空代表程序运行的当前路径
            prefix {str} - 要加载的i18n字典文件前缀
            encoding {str} - 要加载的i18n字典文件的字符编码, 默认为'utf-8'
        @param {kwargs} - 实现类自定义扩展参数
        """
        # 判断初始化是否成功的变量
        self._init_success = False

        # 判断服务器名是否重复
        INIT_SERVER = self.get_init_server_dict()
        if app_name in INIT_SERVER.keys():
            raise RuntimeError(_('server [$1] already exists', app_name))

        # 参数处理
        self._app_name = app_name
        self._server_config = copy.deepcopy(server_config)
        self._support_auths = support_auths
        self._before_server_start = before_server_start
        self._after_server_start = after_server_start
        self._before_server_stop = before_server_stop
        self._after_server_stop = after_server_stop
        self._logger = logger
        self._log_level = log_level
        if logger is None:
            logging.basicConfig()
            self._logger = logging.getLogger(__name__)
            self._logger.level = logging.INFO
        self._kwargs = kwargs

        # 加载多国语言
        self._load_i18n_dict(
            os.path.join(os.path.dirname(__file__), 'i18n'), 'socket_server', 'utf-8'
        )  # 框架默认的语言文件
        _load_i18n_para = load_i18n_para
        if _load_i18n_para is not None:
            _path = _load_i18n_para.get('path', '')
            _prefix = _load_i18n_para.get('prefix', '')
            _encoding = _load_i18n_para.get('encoding', 'utf-8')
            if _prefix != '':
                self._load_i18n_dict(_path, _prefix, _encoding)

        # 内部参数
        self._status = EnumServerRunStatus.Stop  # 服务状态
        self._status_lock = threading.RLock()  # 服务状态变更的锁
        self._start_begin_time = datetime.datetime.now()  # 服务启动开始时间
        self._last_start_result = None  # 启动如果失败, 存放服务线程中返回的失败原因结果对象
        self._stop_begin_time = datetime.datetime.now()  # 服务关闭开始时间
        self._dynamic_var = {}  # 存放动态内部变量的字典, 方便实现类进行变量定义

        # 服务路由管理, key为service_uri, value为{'handler': 执行函数对象, 'kwargs': 添加服务时的kwargs传参}
        self._service_router = {}

        # 标准化服务初始配置
        self._std_init_config()

        # 将自己加入已初始化的服务器对象全局变量
        INIT_SERVER[self._app_name] = self

        # 标注初始化成功
        self._init_success = True

    def __del__(self):
        """
        析构函数
        """
        # 将自己从已初始化的服务器对象全局变量中移除
        if self._init_success:
            INIT_SERVER = self.get_init_server_dict()
            INIT_SERVER.pop(self._app_name, None)

            # 判断是否服务还在运行, 强制关闭服务
            if self._status == EnumServerRunStatus.Running:
                AsyncTools.sync_run_coroutine(
                    self.stop(is_force=True)
                )

    #############################
    # 其他公共属性及函数
    #############################
    @property
    def status(self) -> EnumServerRunStatus:
        """
        获得服务器运行状态

        @property {EnumServerRunStatus} - 返回运行状态
        """
        return self._status

    @property
    def app_name(self) -> str:
        """
        获取应用名
        @property {str}
        """
        return self._app_name

    async def start(self, is_asyn: bool = False, sleep_time: float = 0.5, **kwargs) -> CResult:
        """
        启动服务
        (可以为同步或异步函数)

        @param {bool} is_asyn=False - 是否异步处理, 如果是则直接返回, 同步则阻塞处理直到服务停止
        @param {float} sleep_time=0.5 - 同步处理的每次循环检测状态的睡眠时间, 单位为秒
        @param {kwargs} - 实现类自定义的扩展参数

        @returns {CResult} - 异步情况返回启动结果, result.code: '00000'-成功, '21401'-服务不属于停止状态, 其他-异常
        """
        _result = CResult(code='00000')  # 成功
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER-STARTING][NAME:%s]%s: ' % (
                self._app_name, _('start server error')
            ),
            force_log_level=logging.ERROR
        ):
            # 先获取锁，拿到最准确的服务状态
            self._status_lock.acquire()
            try:
                if self.status != EnumServerRunStatus.Stop:
                    # 不属于停止状态，不能启动
                    _temp_result = CResult(code='21401')  # 服务启动失败-服务已启动
                    self._logger.log(
                        self._log_level,
                        '[SER-STARTING][NAME:%s]%s' % (self._app_name, _temp_result.msg))
                    return _temp_result

                # 执行启动服务的动作，通过线程方式启动
                self._start_begin_time = datetime.datetime.now()
                self._server_status_change(EnumServerRunStatus.WaitStart)
                _server_thread = threading.Thread(
                    target=self._start_server_thread_fun, args=(1,), name='Thread-Server-Main'
                )
                _server_thread.setDaemon(True)
                _server_thread.start()
            finally:
                # 释放锁
                self._status_lock.release()

        if not _result.is_success():
            # 执行出现错误, 直接返回
            return _result

        # 等待服务启动完成
        while self.status == EnumServerRunStatus.WaitStart:
            await asyncio.sleep(sleep_time)

        if self._status != EnumServerRunStatus.Running:
            # 服务线程中出现启动异常
            return self._last_start_result

        if is_asyn:
            # 异步模式, 直接返回
            return _result

        # 同步模式, 循环等待结束
        while self._status == EnumServerRunStatus.Running:
            await asyncio.sleep(sleep_time)

        return _result

    async def stop(self, overtime: float = 0, sleep_time: int = 0.5, **kwargs) -> CResult:
        """
        停止服务运行
        (可以为同步或异步函数)

        @param {float} overtime=0 - 等待超时时间, 单位为秒, 0代表一直不超时
        @param {float} sleep_time=0.5 - 每次等待睡眠时间, 单位为秒
        @param {kwargs} - 实现类自定义的扩展参数
            is_force {bool} - 是否强制关闭服务(有未正常关闭的风险), 默认为False

        @returns {CResult} - 停止结果, result.code: '00000'-成功, '21402'-服务停止失败-服务已关闭,
            '31005'-执行超时, 29999'-其他系统失败
        """
        self._stop_begin_time = datetime.datetime.now()
        _is_force = kwargs.get('is_force', False)
        _result = CResult(code='00000')  # 成功
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER-STOPING][NAME:%s]%s: ' % (
                self._app_name, _('stop service error')),
            force_log_level=logging.ERROR
        ):
            self._status_lock.acquire()
            try:
                _status = EnumServerRunStatus.WaitStop
                if _is_force:
                    _status = EnumServerRunStatus.ForceStop

                if self._status == EnumServerRunStatus.Running:
                    # 运行状态，处理设置等待关闭状态
                    self._logger.log(
                        self._log_level,
                        '[SER-STOPING][NAME:%s]%s' % (self._app_name, _('server stoping')))
                    self._status = _status
                elif self._status == EnumServerRunStatus.WaitStop \
                        and _status == EnumServerRunStatus.ForceStop:
                    self._logger.log(
                        self._log_level,
                        '[SER-STOPING][NAME:%s]%s' % (self._app_name, _('server force stoping')))
                    self._status = _status
                else:
                    # 不属于运行状态，不能处理
                    _temp_result = CResult(code='21402')  # 服务停止失败-服务已关闭
                    self._logger.log(
                        self._log_level,
                        '[SER-STOPING][NAME:%s]%s' % (self._app_name, _temp_result.msg))
                    return _temp_result
            finally:
                self._status_lock.release()

        # 等待服务关闭
        _begin_time = datetime.datetime.now()  # 记录等待开始时间
        while True:
            if self._status == EnumServerRunStatus.Stop:
                break

            if overtime > 0 and (datetime.datetime.now() - _begin_time).total_seconds() > overtime:
                _result = CResult(code='31005')  # 执行超时
                break

            # 等待下一次检查
            await asyncio.sleep(sleep_time)

        # 返回结果
        return _result

    async def add_service(self, service_uri: str, handler: Callable, **kwargs) -> CResult:
        """
        添加请求处理服务
        (可以为同步或异步函数)

        @param {str} service_uri - 服务唯一标识, 例如服务名或url路由
        @param {Callable} handler - 请求处理函数, 应可同时支持同步或异步函数
            注: 由实现类自定义请求函数的要求
        @param {kwargs}  - 实现类的自定义扩展参数

        @returns {CResult} - 添加服务结果, result.code: '00000'-成功, '21405'-服务名已存在, 其他-异常
        """
        _result = CResult(code='00000')  # 成功
        with ExceptionTool.ignored_cresult(
            result_obj=_result, logger=self._logger,
            self_log_msg='[SER][NAME:%s]%s: ' % (self._app_name, _('add service error'))
        ):
            if service_uri in self._service_router.keys():
                # 服务名已存在, 返回错误
                _result = CResult(code='21405', i18n_msg_paras=(service_uri, ))
            else:
                self._service_router[service_uri] = {
                    'handler': handler, 'kwargs': kwargs
                }

        return _result

    async def remove_service(self, service_uri: str, **kwargs) -> CResult:
        """
        移除请求处理服务
        (可以为同步或异步函数)

        @param {str} service_uri - 服务唯一标识, 例如服务名或url路由
        @param {kwargs}  - 实现类的自定义扩展参数

        @returns {CResult} - 添加服务结果, result.code: '00000'-成功, '21403'-服务不存在, 其他-异常
        """
        _result = CResult(code='00000')  # 成功
        with ExceptionTool.ignored_cresult(
            result_obj=_result, logger=self._logger,
            self_log_msg='[SER][NAME:%s]%s: ' % (self._app_name, _('remove service error'))
        ):
            _paras = self._service_router.pop(service_uri, None)
            if _paras is None:
                # 服务名不存在, 返回错误
                _result = CResult(code='21403', i18n_msg_paras=(service_uri, ))

        return _result

    #############################
    # 内部函数
    #############################

    def _load_i18n_dict(self, path: str, prefix: str, encoding: str):
        """
        装载多国语言字典

        @param {str} path - 要加载的i18n字典文件路径, 如果填空代表程序运行的当前路径
        @param {str} prefix - 要加载的i18n字典文件前缀
        @param {str} encoding - 要加载的i18n字典文件的字符编码, 默认为'utf-8'
        """
        _i18n_obj = get_global_i18n()
        if _i18n_obj is None:
            # 创建对象并加入到全局中
            init_global_i18n()
            _i18n_obj = get_global_i18n()

        # 进行语言装载
        _i18n_obj.load_trans_from_dir(
            trans_file_path=path, trans_file_prefix=prefix, encoding=encoding, append=True
        )

    def _server_status_change(self, status: EnumServerRunStatus):
        """
        通用的服务器状态修改函数

        @param {EnumServerRunStatus} status - 要修改的服务器状态
        """
        self._status_lock.acquire()
        try:
            self._status = status
        finally:
            self._status_lock.release()

    def _start_server_thread_fun(self, tid):
        """
        启动服务处理主线程, 本线程结束就代表服务停止

        @param {int} tid - 线程id

        """
        _result = CResult(code='00000')  # 成功
        _server_info = None
        with ExceptionTool.ignored_cresult(
            result_obj=_result, logger=self._logger,
            self_log_msg='[SER-STARTING][NAME:%s]%s: ' % (self._app_name, _('start server error'))
        ):
            # 统一的异常处理
            self._logger.log(
                self._log_level,
                '[SER-STARTING][NAME:%s]%s' % (self._app_name, _('server starting'))
            )

            # 执行before_server_start
            if self._before_server_start is not None:
                AsyncTools.sync_run_coroutine(self._before_server_start(self))

            # 执行真正服务初始化处理，执行通过则代表启动成功
            self._last_start_result = AsyncTools.sync_run_coroutine(self._real_server_initialize(tid))
            if self._last_start_result.code != '00000':
                # 启动失败，登记了日志，修改状态为未启动，退出
                self._logger.log(
                    logging.ERROR,
                    ('[SER-STARTING][NAME:%s][USE:%ss]%s: %s - %s' % (
                        self._app_name,
                        str((datetime.datetime.now() - self._start_begin_time).total_seconds()),
                        _('start server error'), self._last_start_result.code, self._last_start_result.msg))
                )
                self._server_status_change(EnumServerRunStatus.Stop)
                return

            # 启动成功
            _server_info = self._last_start_result.server_info  # 真正服务启动返回的服务器信息
            self._logger.log(
                self._log_level,
                '[SER-STARTED][NAME:%s][USE:%ss]%s' % (
                    self._app_name,
                    str((datetime.datetime.now() - self._start_begin_time).total_seconds()),
                    _('start server sucess')
                ))
            self._server_status_change(EnumServerRunStatus.Running)

            # 服务启动成功, 执行after_server_start
            if self._after_server_start is not None:
                AsyncTools.sync_run_coroutine(self._after_server_start(self))

            # 开始进入循环处理
            while True:
                if self._status == EnumServerRunStatus.WaitStop:
                    # 收到指令等待停止
                    # 服务关闭前处理函数
                    if self._before_server_stop is not None:
                        AsyncTools.sync_run_coroutine(
                            self._before_server_stop(self)
                        )

                    while True:
                        if self._status == EnumServerRunStatus.ForceStop:
                            # 过程中又被要求强制退出
                            break

                        # 执行服务关闭前的处理函数
                        _prepare_stop_result = AsyncTools.sync_run_coroutine(
                            self._real_server_prepare_stop(tid)
                        )
                        if _prepare_stop_result.code == '00000' and not _prepare_stop_result.is_finished:
                            # 预处理未完成，需要循环处理
                            AsyncTools.sync_run_coroutine(
                                asyncio.sleep(0.1)
                            )
                            continue
                        else:
                            # 预处理已完成，退出
                            break
                    break
                elif self._status == EnumServerRunStatus.ForceStop:
                    # 收到指令马上停止
                    # 服务关闭前处理函数
                    if self._before_server_stop is not None:
                        AsyncTools.sync_run_coroutine(
                            self._before_server_stop(self)
                        )

                    break
                else:
                    # 正常执行一次服务处理函数
                    _run_result = AsyncTools.sync_run_coroutine(
                        self._real_server_accept_and_run(tid, _server_info)
                    )

                    # 判断是否结束服务
                    if _run_result.is_success() and not _run_result.is_finished:
                        continue
                    else:
                        # 根据运行处理函数的返回结果, 结束服务
                        # 服务关闭前处理函数
                        if self._before_server_stop is not None:
                            AsyncTools.sync_run_coroutine(
                                self._before_server_stop(self)
                            )
                        break

        # 线程结束就代表服务已关闭，执行结束处理函数
        AsyncTools.sync_run_coroutine(self._real_server_stop(tid, _server_info))
        self._server_status_change(EnumServerRunStatus.Stop)
        self._logger.log(
            self._log_level,
            '[SER-STOPED][NAME:%s][USE:%ss]%s' % (
                self._app_name, str((datetime.datetime.now() - self._stop_begin_time).total_seconds()),
                _('server stoped')
            ))

        # 服务关闭后执行函数
        if self._after_server_stop is not None:
            AsyncTools.sync_run_coroutine(self._after_server_stop(self))

    #############################
    # 实现类需重载的内部函数
    #############################
    async def _real_server_initialize(self, tid) -> CResult:
        """
        初始化真正的服务端处理对象
        注: 可以在该函数中启动真正的服务, 例如绑定监听端口

        @param {int} tid - 服务线程id

        @returns {CResult} - 启动结果:
            result.code: '00000'-成功, 其他值为失败
            result.server_info: 启动成功后的服务信息, 用于传递到后续的服务处理函数
        """
        _result = CResult(code='00000')  # 成功
        _result.server_info = NullObj()
        with ExceptionTool.ignored_cresult(_result):
            # 注: 重载该函数, 可在该部分实现自定义的服务初始化或启动逻辑
            pass

        # 返回处理结果
        return _result

    async def _real_server_accept_and_run(self, tid, server_info) -> CResult:
        """
        真正的服务获取请求并运行
        注: 支持同步或异步函数, 可以在该函数中获取请求, 并启动线程执行请求处理

        @param {int} tid - 线程id
        @param {Any} server_info - 启动成功后的服务信息

        @returns {CResult} - 处理结果:
            result.code: '00000'-成功, 其他值为失败, 如果为失败将停止服务
            result.is_finished: 指示服务是否已完成, True - 已处理完成可停止服务, False - 未完成, 需循环处理下一个请求
        """
        _result = CResult(code='00000')  # 成功
        _result.is_finished = False
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER][NAME:%s]%s: ' % (
                self._app_name, _('server run error')),
            force_log_level=logging.ERROR
        ):
            # 注: 重载该函数, 可在该部分实现自定义的获取请求及处理逻辑
            pass

        # 返回处理结果
        return _result

    async def _real_server_prepare_stop(self, tid) -> CResult:
        """
        服务关闭前的处理函数
        注: 支持同步或异步函数, 可以在该函数中等待已接入的请求或线程完成处理

        @param {int} tid - 线程id

        @returns {CResult} - 处理结果:
            result.code: '00000'-成功, 其他值为失败
            result.is_finished: 指示停止前的预处理是否已完成, True - 已处理完成可停止服务, False - 未完成, 需循环继续调用本函数
        """
        _result = CResult(code='00000')  # 成功
        _result.is_finished = True
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER-PRE-STOP][NAME:%s]%s: ' % (
                self._app_name, _('stop net server error')),
            force_log_level=logging.ERROR
        ):
            # 注: 重载该函数, 可在该部分实现自定义的服务关闭前处理
            pass

        return _result

    async def _real_server_stop(self, tid, server_info):
        """
        真正服务的关闭处理
        注: 支持同步或异步函数, 可以在该函数中清理处理线程并关闭监听

        @param {int} tid - 线程id
        @param {Any} server_info - 启动成功后的服务信息
        """
        # 子类必须定义该功能
        with ExceptionTool.ignored_all(
            logger=self._logger,
            self_log_msg='[SER-STOP][NAME:%s]%s error: ' % (
                self._app_name, _('stop net server error')
            )
        ):
            # 注: 重载该函数, 可在该部分实现自定义的服务关闭处理
            pass

    def _std_init_config(self):
        """
        标准化服务初始配置
        """
        # 实现类可以通过该函数设置或修改内部参数
        pass


class TcpIpServer(ServerBaseFW):
    """
    TcpIp协议的服务实现
    通讯协议为: 前4个字节为后续报文信息的字节长度(int), 后面报文信息为json字符串的字节数组, 字节编码为utf-8
    注:
    1、通过add_service可以添加service_uri为''的处理函数, 如果请求匹配不到service_uri时将使用该函数进行处理;
    2、add_service的处理函数的定义如下:
        func(net_info, service_uri: str, request: dict) -> dict
            service_uri - 匹配上的服务标识
            request - 获取到的请求信息json字典
            返回值为要返回到socket的字典, 如果无需返回则使用None返回
    """

    def __init__(self, app_name: str, server_config: dict = None, support_auths: dict = {},
            before_server_start=None, after_server_start=None,
            before_server_stop=None, after_server_stop=None, logger=None, log_level: int = logging.INFO,
            load_i18n_para=None, **kwargs):
        """
        构造函数

        @param {str} app_name - 服务器名称
        @param {dict} server_config={} - 服务配置字典
            ip {str} - 主机名或IP地址, 默认为''
            port {int} - 监听端口, 默认为8080
            max_connect {int} - 允许最大连接数, 默认为20
            recv_timeout {float} - 数据接收的超时时间, 单位为秒, 默认为10
            send_timeout {float} - 数据发送的超时时间, 单位为秒, 默认为10
        @param {dict} support_auths={} - 服务器支持的验证对象字典, key为验证对象类型名(可以为类名), value为验证对象实例对象
            注意: 支持的auth对象必须有auth_required这个修饰符函数
        @param {function} before_server_start=None - 服务器启动前执行的函数对象, 传入服务自身(self)
        @param {function} after_server_start=None - 服务器启动后执行的函数对象, 传入服务自身(self)
        @param {function} before_server_stop=None - 服务器关闭前执行的函数对象, 传入服务自身(self)
        @param {function} after_server_stop=None - 服务器关闭后执行的函数对象, 传入服务自身(self)
        @param {Logger} logger=None - 自定义应用逻辑使用的日志对象
        @param {int} log_level=logging.INFO - 一般信息的记录使用的日志级别
        @param {dict} load_i18n_para=None - 要装载的i18n语言文件配置
            path {str} - 要加载的i18n字典文件路径, 如果填空代表程序运行的当前路径
            prefix {str} - 要加载的i18n字典文件前缀
            encoding {str} - 要加载的i18n字典文件的字符编码, 默认为'utf-8'
        @param {kwargs} - 自定义扩展参数
            match_service_uri_func {function} - 获取请求匹配的service_uri的函数, 格式如下:
                func(request: dict) -> str
                其中request为获取到的请求信息字典, 返回所匹配的service_uri字符串
        """
        super().__init__(
            app_name, server_config=server_config, support_auths=support_auths,
            before_server_start=before_server_start,
            after_server_start=after_server_start, before_server_stop=before_server_stop,
            after_server_stop=after_server_stop, logger=logger, log_level=log_level,
            load_i18n_para=load_i18n_para, **kwargs
        )
        self._match_service_uri_func = kwargs.get('match_service_uri_func', None)

    #############################
    # 实现类需重载的内部函数
    #############################
    async def _real_server_initialize(self, tid) -> CResult:
        """
        初始化真正的服务端处理对象
        注: 可以在该函数中启动真正的服务, 例如绑定监听端口

        @param {int} tid - 服务线程id

        @returns {CResult} - 启动结果:
            result.code: '00000'-成功, 其他值为失败
            result.server_info: 启动成功后的服务信息, 用于传递到后续的服务处理函数
        """
        _result = CResult(code='00000')  # 成功
        _result.server_info = NullObj()
        with ExceptionTool.ignored_cresult(_result):
            # 注: 重载该函数, 可在该部分实现自定义的服务初始化或启动逻辑
            # 启动服务，但不接受连接
            self._logger.log(
                self._log_level,
                '[SER-STARTING][NAME:%s]%s:\n%s' % (
                    self._app_name, _('net start parameter'), self._formated_server_config()
                )
            )
            _result = self._start_server_without_accept()
            _result.server_info = _result.server_info

        # 返回处理结果
        return _result

    async def _real_server_accept_and_run(self, tid, server_info) -> CResult:
        """
        真正的服务获取请求并运行
        注: 支持同步或异步函数, 可以在该函数中获取请求, 并启动线程执行请求处理

        @param {int} tid - 线程id
        @param {Any} server_info - 启动成功后的服务信息

        @returns {CResult} - 处理结果:
            result.code: '00000'-成功, 其他值为失败, 如果为失败将停止服务
            result.is_finished: 指示服务是否已完成, True - 已处理完成可停止服务, False - 未完成, 需循环处理下一个请求
        """
        _result = CResult(code='00000')  # 成功
        _result.is_finished = False
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER][NAME:%s]%s: ' % (
                self._app_name, _('server run error')),
            force_log_level=logging.ERROR
        ):
            # 注: 重载该函数, 可在该部分实现自定义的获取请求及处理逻辑
            # 监听下一个连接请求
            _accept_result = await AsyncTools.async_run_coroutine(self._accept_one(server_info))

            if _accept_result.is_success():
                # 获取到一个连接，获取请求信息及处理
                await AsyncTools.async_run_coroutine(
                    self._get_request_and_deal(_accept_result.net_info)
                )
            elif _accept_result.code != '20407':
                # 不是超时的其他获取错误，打印信息
                self._logger.log(
                    logging.ERROR,
                    "[SER][NAME:%s][EX:%s]%s: %s\n%s" % (
                        self._app_name, str(type(_accept_result.error)),
                        _('accept net connection error'), _accept_result.msg,
                        _accept_result.trace_str
                    )
                )
            else:
                # 监听超时, 睡眠一小段时间, 释放cpu
                await asyncio.sleep(0.01)

        # 返回处理结果
        return _result

    async def _real_server_prepare_stop(self, tid) -> CResult:
        """
        服务关闭前的处理函数
        注: 支持同步或异步函数, 可以在该函数中等待已接入的请求或线程完成处理

        @param {int} tid - 线程id

        @returns {CResult} - 处理结果:
            result.code: '00000'-成功, 其他值为失败
            result.is_finished: 指示停止前的预处理是否已完成, True - 已处理完成可停止服务, False - 未完成, 需循环继续调用本函数
        """
        _result = CResult(code='00000')  # 成功
        _result.is_finished = True
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER-PRE-STOP][NAME:%s]%s: ' % (
                self._app_name, _('stop net server error')),
            force_log_level=logging.ERROR
        ):
            # 注: 重载该函数, 可在该部分实现自定义的服务关闭前处理
            # 等待所有连接处理完成
            if len(self._dynamic_var['connect_thread']['list']) > 0:
                _result.is_finished = False

        return _result

    async def _real_server_stop(self, tid, server_info):
        """
        真正服务的关闭处理
        注: 支持同步或异步函数, 可以在该函数中清理处理线程并关闭监听

        @param {int} tid - 线程id
        @param {Any} server_info - 启动成功后的服务信息
        """
        # 子类必须定义该功能
        with ExceptionTool.ignored_all(
            logger=self._logger,
            self_log_msg='[SER-STOP][NAME:%s]%s error: ' % (
                self._app_name, _('stop net server error')
            )
        ):
            # 注: 重载该函数, 可在该部分实现自定义的服务关闭处理
            await AsyncTools.async_run_coroutine(self._stop_server(server_info))

    def _std_init_config(self):
        """
        标准化服务初始配置
        """
        # 实现类可以通过该函数设置或修改内部参数
        if self._server_config.get('ip', None) is None:
            self._server_config['ip'] = ''
        if self._server_config.get('port', None) is None:
            self._server_config['port'] = 8080

        _max_connect = self._server_config.get('max_connect', None)
        if _max_connect is None or _max_connect <= 0:
            _max_connect = 20

    #############################
    # 内部函数
    #############################

    def _formated_server_config(self) -> str:
        """
        获取字符串格式化后的服务配置参数

        @returns {str} - 要输出的格式化后字符串
        """
        return json.dumps(self._server_config, ensure_ascii=False, indent=2)

    def _start_server_without_accept(self) -> CResult:
        """
        启动服务但不接受请求服务
        注: 该方法只做到启动端口层面, 轮询监听不在该方法中实现, 注意该该函数必须捕获并处理异常

        @returns {CResult} - 启动结果:
            result.code: '00000'-成功, 其他值为失败
            result.server_info: 启动后的服务端网络连接信息对象, 该对象将传给后续的监听线程(self._accept_one)
                server_info = NullObj()
                server_info.csocket - socket对象
                server_info.laddr 本地地址, 地址对象, ("IP地址",打开端口)
                server_info.raddr 远端地址, 地址对象, ("IP地址",打开端口)
                server_info.send_timeout 发送超时时间, 单位为毫秒
                server_info.recv_timeout 收取超时时间, 单位为毫秒
        """
        _result = CResult('00000')
        _result.net_info = None
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger,
            self_log_msg='[SER-LIS-STARTING][NAME:%s]%s - %s error: ' % (
                self._app_name, _('net server starting'), _('listen without accept')),
            force_log_level=logging.ERROR
        ):
            # 生成请求执行线程任务变量参数
            self._dynamic_var['connect_thread'] = {
                'id': 1,  # 服务端的链接线程ID序列
                'list': {},  # 服务端正在运行的连接线程列表
                'list_lock': threading.RLock()  # 连接线程列表变更的同步锁
            }

            # 绑定监听端口, 同时返回服务信息
            _server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            _server_socket.setblocking(False)   # 将socket设置为非阻塞. 在创建socket对象后就进行该操作.
            _server_socket.bind((self._server_config['ip'], self._server_config['port']))
            _server_socket.listen(self._server_config.get('max_connect', 20))

            _result.server_info = NullObj()
            _result.server_info.laddr = _server_socket.getsockname()
            _result.server_info.raddr = ('', 0)
            _result.server_info.csocket = _server_socket
            _result.server_info.send_timeout = self._server_config.get('send_timeout', 10.0)
            _result.server_info.recv_timeout = self._server_config.get('recv_timeout', 10.9)

        return _result

    async def _accept_one(self, server_info: NullObj) -> CResult:
        """
        监听接受一个请求并返回
        注: 提供监听并获取到请求连接返回的方法, 注意该该函数必须捕获并处理异常

        @param {NullObj} server_info - 服务端连接信息对象, self._start_server_without_accept 中获取到的结果

        @returns {CResult} - 获取网络连接结果:
            result.code: '00000'-成功, '20407'-获取客户端连接请求超时
            result.net_info: 客户端连接信息对象, 该对象将传给后续单个连接处理的线程
        """
        # 子类必须定义该功能
        _result = CResult('00000')
        _result.net_info = None
        with ExceptionTool.ignored_cresult(
            _result, logger=self._logger, expect=(BlockingIOError), expect_no_log=True,  # 超时不记录日志
            error_map={BlockingIOError: ('20407', None)},
            self_log_msg='[SER-LIS][NAME:%s]%s error: ' % (
                self._app_name, _('accept client connect')),
            force_log_level=None
        ):
            _csocket, _addr = server_info.csocket.accept()  # 接收客户端连接，返回客户端和地址
            _csocket.setblocking(False)   # 将socket设置为非阻塞. 在创建socket对象后就进行该操作.
            _result.net_info = NullObj()
            _result.net_info.csocket = _csocket
            _result.net_info.raddr = _addr
            _result.net_info.laddr = _csocket.getsockname()
            _result.net_info.send_timeout = self._server_config.get('send_timeout', 10.0)
            _result.net_info.recv_timeout = self._server_config.get('recv_timeout', 10.0)

            self._logger.log(
                self._log_level,
                '[SER-LIS][NAME:%s]%s: %s - %s' % (
                    self._app_name, _('accept one client connection'), str(_addr), str(_csocket)
                )
            )

        return _result

    async def _get_request_and_deal(self, net_info):
        """
        获取请求信息并进行处理
        注: 支持同步和异步函数, 可以启动一个新线程进行处理, 注意函数需捕获并处理异常

        @param {NullObj} net_info - 连接信息对象, self._accept_one 中获取到的结果
        """
        # 子类必须定义该功能
        with ExceptionTool.ignored_all(
            logger=self._logger,
            self_log_msg='[SER-DEAL][NAME:%s]%s error: ' % (
                self._app_name, _('net server connect deal threading error')
            )
        ):
            # 放入实际的获取请求数据和处理的逻辑代码
            # 创建线程
            self._dynamic_var['connect_thread']['list_lock'].acquire()
            try:
                self._dynamic_var['connect_thread']['id'] += 1
                _thread_id = self._dynamic_var['connect_thread']['id']
                _new_thread = threading.Thread(
                    target=self._connect_deal_thread_func,
                    args=(_thread_id, net_info),
                    name='Thread-ConnectDeal' + str(_thread_id)
                )
                # 将线程添加到队列
                self._dynamic_var['connect_thread']['list'][_thread_id] = _new_thread
            finally:
                self._dynamic_var['connect_thread']['list_lock'].release()

            _new_thread.setDaemon(True)
            _new_thread.start()

    async def _stop_server(self, server_info: NullObj):
        """
        关闭服务吹
        注: 支持同步和异步函数, 提供关闭服务的方法

        @param {NullObj} server_info - 服务端连接信息对象, self._start_server_without_accept 中获取到的结果
        """
        # 放入实际的关闭服务逻辑
        # 清除连接线程
        self._dynamic_var['connect_thread']['list_lock'].acquire()
        try:
            self._dynamic_var['connect_thread']['list'].clear()
        finally:
            self._dynamic_var['connect_thread']['list_lock'].release()

        # 关闭网络连接
        server_info.csocket.close()

    #############################
    # 处理线程相关函数
    #############################

    def _connect_deal_thread_func(self, thread_id: int, net_info):
        """
        连接处理线程

        @param {int} thread_id - 线程id
        @param {NullObj} net_info - 连接信息对象, self._accept_one 中获取到的结果
        """
        try:
            with ExceptionTool.ignored_all(
                logger=self._logger,
                self_log_msg='[SER-DEAL][NAME:%s]%s: ' % (self._app_name, _(
                    'net server connect deal threading error')),
                force_log_level=logging.ERROR
            ):
                # 注: 重载该函数, 可在该部分实现自定义请求处理逻辑
                # 获取请求标准信息
                _request_dict = AsyncTools.sync_run_coroutine(
                    self._get_std_request(net_info)
                )
                if _request_dict is None:
                    return

                # 获取匹配的处理函数
                _match_service_uri = ''
                if self._match_service_uri_func is not None:
                    _match_service_uri = AsyncTools.sync_run_coroutine(
                        self._match_service_uri_func(_request_dict)
                    )

                _deal_fun_para = self._service_router.get(_match_service_uri, None)
                if _deal_fun_para is None:
                    _deal_fun_para = self._service_router.get('', None)

                if _deal_fun_para is None:
                    raise ModuleNotFoundError(_('service "[$1]" not found', _match_service_uri))

                # 执行处理
                _resp_dict = AsyncTools.sync_run_coroutine(
                    _deal_fun_para['handler'](net_info, _match_service_uri, _request_dict)
                )

                # 处理返回
                if _resp_dict is not None:
                    _data = json.dumps(_resp_dict, ensure_ascii=False).encode(encoding='utf-8')
                    _data = NetTool.int_to_bytes(len(_data), signed=False) + _data
                    _send_result = AsyncTools.sync_run_coroutine(
                        SocketTool.send_data(net_info, _data)
                    )
                    if not _send_result.is_success():
                        # 记录失败日志
                        self._logger.error('%s: %s' % (_('send data to remote error'), str(_send_result)))
        finally:
            # 关闭网络连接
            AsyncTools.sync_run_coroutine(
                SocketTool.close(net_info)
            )

            # 结束处理, 删除线程清单
            self._dynamic_var['connect_thread']['list_lock'].acquire()
            try:
                self._dynamic_var['connect_thread']['list'].pop(thread_id, None)
            finally:
                self._dynamic_var['connect_thread']['list_lock'].release()

    async def _get_std_request(self, net_info) -> dict:
        """
        获取标准请求对象

        @param {NullObj} net_info - 需要关闭的网络连接信息对象
            net_info.csocket - socket对象
            net_info.laddr 本地地址,地址对象("IP地址",打开端口)
            net_info.raddr 远端地址,地址对象("IP地址",打开端口)
            net_info.send_timeout 发送超时时间, 单位为秒
            net_info.recv_timeout 收取超时时间, 单位为秒

        @returns {dict} - 返回请求处理后的字典
        """
        _request_dict = None
        # 获取报文信息长度
        _result = await AsyncTools.async_run_coroutine(
            SocketTool.recv_data(net_info, {'recv_len': 4})
        )
        if _result.is_success():
            _msg_len = NetTool.bytes_to_int(_result.data, signed=False)

            # 获取报文信息
            _result = await AsyncTools.async_run_coroutine(
                SocketTool.recv_data(net_info, {'recv_len': _msg_len})
            )
            if _result.is_success():
                _request_dict = json.loads(_result.data)

        if _request_dict is not None:
            # 正常获取到信息
            return _request_dict
        else:
            # 获取失败, 打印错误日志
            self._logger.error('%s: %s' % (_('recv data from remote error'), str(_result)))
            return None


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名: %s  -  %s\n'
           '作者: %s\n'
           '发布日期: %s\n'
           '版本: %s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))
